@@ -2,13 +2,16 @@ package Test::BrewBuild;
 use strict;
 use warnings;
 
+use Data::Dumper;
 use File::Temp;
 use Logging::Simple;
+use Test::BrewBuild::BrewCommands;
 use Test::BrewBuild::Plugin;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 my $log;
+my $bcmd;
 
 sub new {
     my ($class, %args) = @_;
@@ -18,11 +21,10 @@ sub new {
     $log = $self->_create_log($args{debug});
     $log->_7("in new(), constructing " . __PACKAGE__ . " object");
 
+    $bcmd = Test::BrewBuild::BrewCommands->new($log);
     my $exec_plugin_name = $args{plugin} ? $args{plugin} : $ENV{TBB_PLUGIN};
 
-    if ($exec_plugin_name){
-        $log->_7("plugin param set to: $exec_plugin_name");
-    }
+    $log->_7("plugin param set to: $exec_plugin_name") if $exec_plugin_name;
 
     $exec_plugin_name = $self->plugin($exec_plugin_name);
 
@@ -36,19 +38,12 @@ sub new {
 sub perls_available {
     my ($self, $brew_info) = @_;
 
-    $log->_7("in perls_available()");
+    my $log = $log->child('perls_available');
 
-    my @perls_available = $self->is_win
-        ? $brew_info =~ /(\d\.\d+\.\d+_\d+)/g
-        : $brew_info =~ /(perl-\d\.\d+\.\d+)/g;
+    my @perls_available = $bcmd->available($brew_info);
 
     $log->_7("perls available: " . join ', ', @perls_available);
 
-    if ($self->is_win){
-        for (@perls_available){
-            s/perl-//;
-        }
-    }
     return @perls_available;
 }
 sub perls_installed {
@@ -57,9 +52,7 @@ sub perls_installed {
     my $log = $log->child('perls_installed');
     $log->_7("checking perls installed");
 
-    return $self->is_win
-        ? $brew_info =~ /(\d\.\d{2}\.\d(?:_\d{2}))(?!=_)\s+\[installed\]/ig
-        : $brew_info =~ /i.*?(perl-\d\.\d+\.\d+)/g;
+    return $bcmd->installed($brew_info);
 }
 sub instance_remove {
     my ($self, @perls_installed) = @_;
@@ -71,9 +64,7 @@ sub instance_remove {
         $log->_5("removing previous installs...");
     }
 
-    my $remove_cmd = $self->is_win
-        ? 'berrybrew remove'
-        : 'perlbrew uninstall';
+    my $remove_cmd = $bcmd->remove;
 
     $log->_7("using $remove_cmd remove command");
 
@@ -94,26 +85,17 @@ sub instance_remove {
     $log->_5("removal of existing perl installs complete...");
 }
 sub instance_install {
-    my $self = shift;
-    my $count = shift;
-    my $perls_available = shift;
-    my $perls_installed = shift;
+    my ($self, $new, $perls_available, $perls_installed) = @_;
 
     my $log = $log->child('instance_install');
 
-    my $install_cmd = $self->is_win
-        ? 'berrybrew install'
-        : 'perlbrew install --notest -j 4';
-
-    $log->_7("install cmd set to $install_cmd");
+    my $install_cmd = $bcmd->install;
 
     my @new_installs;
 
     if ($self->{args}{version}->[0]){
         for my $version (@{ $self->{args}{version} }){
-            $version = $self->is_win
-                ? $version
-                : "perl-$version";
+            $version = $bcmd->version($_);
 
             if (grep { $version eq $_ } @{ $perls_installed }){
                 $log->_5("$version is already installed... skipping");
@@ -123,20 +105,21 @@ sub instance_install {
         }
     }
     else {
-        if ($count){
+        if ($new){
 
-            $log->_7("looking to install $count perl instances");
+            $log->_7("looking to install $new perl instance(s)");
+
+            while ($new > 0){
 
-            while ($count > 0){
                 my $candidate = $perls_available->[rand @{ $perls_available }];
+
                 if (grep { $_ eq $candidate } @{ $perls_installed }) {
-                    if ($self->{args}{debug}) {
-                        warn "$candidate already installed... skipping\n";
-                    }
+                    $log->_5( "$candidate already installed... skipping" );
                     next;
                 }
+
                 push @new_installs, $candidate;
-                $count--;
+                $new--;
             }
         }
     }
@@ -145,8 +128,8 @@ sub instance_install {
         $log->_5("preparing to install..." . join ', ', @new_installs);
 
         for my $ver (@new_installs){
-            $log->_5("installing $ver...");
-            $log->_7("...using cmd $install_cmd");
+            $log->_0("installing $ver...");
+            $log->_7("...using cmd: $install_cmd");
             `$install_cmd $ver`;
         }
     }
@@ -180,7 +163,7 @@ sub results {
         my $res;
 
         if (/Result:\s+(PASS)/){
-            $log->_7("$ver passed...");
+            $log->_7("$ver PASSED...");
             $res = $1;
         }
         else {
@@ -200,9 +183,8 @@ sub results {
 }
 sub run {
     my $self = shift;
-    my $count = shift;
 
-    $count = 0 if ! $count;
+    my $new = defined $self->{args}{new} ? $self->{args}{new} : 0;
 
     my $log = $log->child('run');
     $log->_7("commencing run()");
@@ -211,15 +193,16 @@ sub run {
 
     my @perls_available = $self->perls_available($brew_info);
 
-    $count = scalar @perls_available if $count < 0;
+    $new = scalar @perls_available if $new < 0;
 
     my @perls_installed = $self->perls_installed($brew_info);
 
     $log->_5("installed perls: " . join ', ', @perls_installed);
 
     $self->instance_remove(@perls_installed) if $self->{args}{remove};
-    if ($count) {
-        $self->instance_install($count, \@perls_available, \@perls_installed);
+
+    if ($new) {
+        $self->instance_install($new, \@perls_available, \@perls_installed);
     }
 
     $brew_info = $self->brew_info;
@@ -233,14 +216,7 @@ sub run {
 
     $self->results();
 }
-sub is_win {
-    my $log = $log->child("is_win");
-    my $is_win = ($^O =~ /Win/) ? 1 : 0;
-    $log->_7("is Windows: $is_win");
-    return $is_win;
-}
 sub exec {
-    my (@a, @b);
     my $self = shift;
 
     my $log = $log->child('exec');
@@ -264,22 +240,28 @@ sub exec {
 
     $log->_7("temp file handle closed");
 
-    my $brew = $self->is_win ? 'berrybrew' : 'perlbrew';
+    my $brew = $bcmd->brew;
 
-    $log->_7("exec'ing: $brew exec perl $fname");
+    if ($self->{args}{on}){
+        my $vers = join ',', @{ $self->{args}{on} };
+        $log->_7("versions to run on: $vers");
+        $log->_7("exec'ing: $brew exec --with $vers perl $fname");
 
-    return `$brew exec perl $fname`;
+        return `$brew exec --with $vers perl $fname`;
+    }
+    else {
+        $log->_7("exec'ing: $brew exec perl $fname");
+        return `$brew exec perl $fname`;
+    }
 }
 sub brew_info {
     my $self = shift;
 
     my $log = $log->child('brew_info');
 
-    my $brew_info = $self->is_win
-        ? `berrybrew available`
-        : `perlbrew available`;
+    my $brew_info = $bcmd->available;
 
-    $log->_7("brew info set to:\n$brew_info");
+    $log->_7("brew info set to:\n$brew_info") if $brew_info;
 
     return $brew_info;
 }
@@ -307,12 +289,15 @@ sub log {
     $self->{log}->_7(ref($self) ." class/obj retrieving a log object");
     return $self->{log};
 }
+sub is_win {
+    my $is_win = ($^O =~ /Win/) ? 1 : 0;
+    return $is_win;
+}
 1;
 
 =head1 NAME
 
-Test::BrewBuild - Perl/Berry brew unit testing automation across installed perl
-versions (Windows and Unix).
+Test::BrewBuild - Backend module for the L<brewbuild> command-line program.
 
 =for html
 <a href="http://travis-ci.org/stevieb9/p5-test-brewbuild"><img src="https://secure.travis-ci.org/stevieb9/p5-test-brewbuild.png"/>
@@ -321,14 +306,14 @@ versions (Windows and Unix).
 
 =head1 DESCRIPTION
 
-This module is the backend for the C<brewbuild> script that is accompanied
+This module is the backend for the C<brewbuild> script that is accompanied by
 this module. For almost all cases, you should be using that script instead of
 using this module directly (so go read that documentation for real use cases),
 as this module is just a helper for the installed script, and isn't designed
 for end-user use.
 
-It facilitates perform your unit tests across all of your Perlbrew (Unix) or
-Berrybrew (Windows) Perl instances.
+It provides you the ability to perform your unit tests across all of your
+Perlbrew (Unix) or Berrybrew (Windows) Perl instances.
 
 For Windows, you'll need to install Berrybrew (see L<SEE ALSO> for details).
 For Unix, you'll need Perlbrew.
@@ -347,10 +332,12 @@ All unit tests are run against all installed instances.
     # default settings
 
     my %args = (
-        debug   => 0,
-        remove  => 0,
-        version => '',
-        count   => 0,
+        debug   => undef,
+        remove  => undef,
+        version => undef,
+        new     => undef,
+        plugin  => undef,
+        on      => undef,
     );
 
     my $bb = Test::BrewBuild->new(%args);
@@ -363,7 +350,7 @@ All unit tests are run against all installed instances.
 
     $bb->instance_remove;
 
-    # install a specific version (uses 'version' param, or 'count'. If 'count'
+    # install a specific version (uses 'version' param, or 'new'. If 'new'
     # is set to a positive integer, we'll randomly install that many instances)
 
     $bb->instance_install;
@@ -387,6 +374,10 @@ can't be located (or it contains errors), we fall back to the default bundled
 L<Test::BrewBuild::Plugin::DefaultExec> (which is the canonical example for
 writing new plugins).
 
+Note that you can send in a custom plugin C<*.pm> filename to plugin as opposed
+to a module name if the module isn't installed. If the file isn't in the
+current working directory, send in the relative or full path.
+
 =head2 perls_available
 
 Returns an array containing all perls available, whether already installed or
@@ -399,7 +390,7 @@ setup.
 
 =head2 instance_install
 
-If 'version' param is set, will install that specific version. If 'count' param
+If 'version' param is set, will install that specific version. If 'new' param
 is set to a positive integer, will install that many random versions of perl.
 
 =head2 instance_remove
